/* ccgo: igsconn.cc
 *
 * IgsConn class definition
 * 
 * Copyright 2002 Chun-Chung Chen <cjj@u.washington.edu>
 *
 * This program is free software; you can redistribute it and'or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */
#include "igsconn.hh"
extern "C" {
#include <sys/types.h>
#include <sys/fcntl.h>
#include <sys/wait.h>
#include <unistd.h>
}

using namespace std;

IgsConn::IgsConn()
{
	// fork a child
	int c2p[2];
	int p2c[2];
	pipe(c2p);
	pipe(p2c);
	child_pid = fork();
	if (child_pid == -1) throw "Can't fork!";
	if (child_pid == 0) {
		close(0); // close cin
		dup(p2c[0]); // from parent
		close(1); // close cout
		dup(c2p[1]); // to parent
		close(p2c[0]);
		close(p2c[1]);
		close(c2p[0]);
		close(c2p[1]);
		execl("/usr/bin/telnet", "telnet", "igs.joyjoy.net",
		      "6969", 0);
	}
	close(p2c[0]);
	to_child = p2c[1];
	from_child = c2p[0];
	close(c2p[1]);
	// set nonblock on reading child
	fcntl(from_child, F_SETFL, O_NONBLOCK);

	con_grab_line = Gtk::Main::input.connect
		(slot(this, & IgsConn::grab_line), from_child, GDK_INPUT_READ);
	con_timeup = Gtk::Main::timeout.connect
		(slot(this, & IgsConn::timeup), 100);
}

IgsConn::~IgsConn()
{
	con_grab_line.disconnect();
	con_timeup.disconnect();
	if (child_pid != 0) {
		kill(child_pid, SIGHUP);
		int s;
		waitpid(child_pid, & s, 0);
	}
}

void IgsConn::send_line(const string & s)
{
	write(to_child, s.c_str(), s.length());
	write(to_child, "\n", 1);
}

bool IgsConn::more_line()
{
	return more_to_come;
}

void IgsConn::grab_line(int, GdkInputCondition)
{
	/*
	  should grab the text line by line...
	 */
	int s;
	// Check if child died
	if (waitpid(child_pid, & s, WNOHANG) != 0) {
		// child is dead -> self-destruct
		// std::cerr << "Child dies!" << std::endl;
		child_pid = 0;
		conn_lost.emit();
		return;
	}
	char b[256];
	// try to read something
	s = read(from_child, b, 255);
	if (s <= 0) return;
	idle_time = 0;
	// we got something. try process it and keep reading
	do {
		b[s] = 0; // put a stopper at the end of the data
		char * sb = b; // sb is the start of the data to be processed
		int p = 1;
		for (int i = 0; sb[i] != 0; i ++) {
			if (sb[i] == '\n') {
				// break it at new line
				if (p < s) more_to_come = true;
				else more_to_come = false;
				line_buffer.append(sb, i);
				line_ready.emit(line_buffer);
				line_buffer = "";
				sb += i + 1; // move the start forward
				i = - 1;
			} else if (iscntrl(sb[i])) sb[i] = ' ';
			//remove control characters^^^^^^^^^^^
			p ++;
		}
		line_buffer.append(sb); // put the rest in the line buffer
		s = read(from_child, b, 255); // try to read more
	} while (s > 0);
	return;
}

gint IgsConn::timeup()
{
	if (idle_time < 30000) {
		idle_time += 100;
		if (idle_time == 1000 && line_buffer != "") {
			line_ready.emit(line_buffer);
			line_buffer = "";
		}
	}
	return true;
}
